@@ -16,7 +16,10 @@ use std::{
     hash::Hash,
     mem,
     num::{NonZeroU8, NonZeroUsize},
-    ops::{Add, AddAssign, Deref, DerefMut, Not, Sub, SubAssign},
+    ops::{
+        Add, AddAssign, BitAnd, BitAndAssign, BitOr, BitOrAssign, Deref, DerefMut, Not, Sub,
+        SubAssign,
+    },
 };
 
 #[cfg(not(feature = "no_float"))]
@@ -201,13 +204,7 @@ pub struct AST {
 impl Default for AST {
     #[inline(always)]
     fn default() -> Self {
-        Self {
-            source: None,
-            body: Default::default(),
-            functions: Default::default(),
-            #[cfg(not(feature = "no_module"))]
-            resolver: None,
-        }
+        Self::empty()
     }
 }
 
@@ -227,6 +224,18 @@ impl AST {
             resolver: None,
         }
     }
+    /// Create an empty [`AST`].
+    #[inline]
+    #[must_use]
+    pub(crate) fn empty() -> Self {
+        Self {
+            source: None,
+            body: Default::default(),
+            functions: Default::default(),
+            #[cfg(not(feature = "no_module"))]
+            resolver: None,
+        }
+    }
     /// Create a new [`AST`] with a source name.
     #[inline(always)]
     #[must_use]
@@ -837,20 +846,6 @@ impl fmt::Debug for Ident {
     }
 }
 
-/// _(internals)_ A type encapsulating the mode of a `return`/`throw` statement.
-/// Exported under the `internals` feature only.
-///
-/// # Volatile Data Structure
-///
-/// This type is volatile and may change.
-#[derive(Debug, Eq, PartialEq, Clone, Copy, Hash)]
-pub enum ReturnType {
-    /// `return` statement.
-    Return,
-    /// `throw` statement.
-    Exception,
-}
-
 /// _(internals)_ An [`AST`] node, consisting of either an [`Expr`] or a [`Stmt`].
 /// Exported under the `internals` feature only.
 ///
@@ -859,7 +854,9 @@ pub enum ReturnType {
 /// This type is volatile and may change.
 #[derive(Debug, Clone, Hash)]
 pub enum ASTNode<'a> {
+    /// A statement ([`Stmt`]).
     Stmt(&'a Stmt),
+    /// An expression ([`Expr`]).
     Expr(&'a Expr),
 }
 
@@ -875,7 +872,17 @@ impl<'a> From<&'a Expr> for ASTNode<'a> {
     }
 }
 
-/// _(internals)_ A statements block.
+impl ASTNode<'_> {
+    /// Get the [`Position`] of this [`ASTNode`].
+    pub const fn position(&self) -> Position {
+        match self {
+            ASTNode::Stmt(stmt) => stmt.position(),
+            ASTNode::Expr(expr) => expr.position(),
+        }
+    }
+}
+
+/// _(internals)_ A scoped block of statements.
 /// Exported under the `internals` feature only.
 ///
 /// # Volatile Data Structure
@@ -904,18 +911,12 @@ impl StmtBlock {
     pub fn len(&self) -> usize {
         self.0.len()
     }
-    /// Get the position of this statements block.
+    /// Get the position (location of the beginning `{`) of this statements block.
     #[inline(always)]
     #[must_use]
     pub const fn position(&self) -> Position {
         self.1
     }
-    /// Get the statements of this statements block.
-    #[inline(always)]
-    #[must_use]
-    pub fn statements_mut(&mut self) -> &mut StaticVec<Stmt> {
-        &mut self.0
-    }
 }
 
 impl Deref for StmtBlock {
@@ -956,7 +957,7 @@ impl From<StmtBlock> for Stmt {
 pub struct OptionFlags(u8);
 
 impl OptionFlags {
-    /// Does this [`BitOptions`] contain a particular option flag?
+    /// Does this [`OptionFlags`] contain a particular option flag?
     #[inline(always)]
     #[must_use]
     pub const fn contains(self, flag: Self) -> bool {
@@ -967,15 +968,17 @@ impl OptionFlags {
 impl Not for OptionFlags {
     type Output = Self;
 
+    /// Return the negation of the [`OptionFlags`].
     #[inline(always)]
     fn not(self) -> Self::Output {
-        Self(!self.0)
+        Self(!self.0) & AST_OPTION_FLAGS::AST_OPTION_ALL
     }
 }
 
 impl Add for OptionFlags {
     type Output = Self;
 
+    /// Return the union of two [`OptionFlags`].
     #[inline(always)]
     fn add(self, rhs: Self) -> Self::Output {
         Self(self.0 | rhs.0)
@@ -983,15 +986,35 @@ impl Add for OptionFlags {
 }
 
 impl AddAssign for OptionFlags {
+    /// Add the option flags in one [`OptionFlags`] to another.
     #[inline(always)]
     fn add_assign(&mut self, rhs: Self) {
         self.0 |= rhs.0
     }
 }
 
+impl BitOr for OptionFlags {
+    type Output = Self;
+
+    /// Return the union of two [`OptionFlags`].
+    #[inline(always)]
+    fn bitor(self, rhs: Self) -> Self::Output {
+        Self(self.0 | rhs.0)
+    }
+}
+
+impl BitOrAssign for OptionFlags {
+    /// Add the option flags in one [`OptionFlags`] to another.
+    #[inline(always)]
+    fn bitor_assign(&mut self, rhs: Self) {
+        self.0 |= rhs.0
+    }
+}
+
 impl Sub for OptionFlags {
     type Output = Self;
 
+    /// Return the difference of two [`OptionFlags`].
     #[inline(always)]
     fn sub(self, rhs: Self) -> Self::Output {
         Self(self.0 & !rhs.0)
@@ -999,12 +1022,31 @@ impl Sub for OptionFlags {
 }
 
 impl SubAssign for OptionFlags {
+    /// Remove the option flags in one [`OptionFlags`] from another.
     #[inline(always)]
     fn sub_assign(&mut self, rhs: Self) {
         self.0 &= !rhs.0
     }
 }
 
+impl BitAnd for OptionFlags {
+    type Output = Self;
+
+    /// Return the intersection of two [`OptionFlags`].
+    #[inline(always)]
+    fn bitand(self, rhs: Self) -> Self::Output {
+        Self(self.0 & !rhs.0)
+    }
+}
+
+impl BitAndAssign for OptionFlags {
+    /// Keep only the intersection of one [`OptionFlags`] with another.
+    #[inline(always)]
+    fn bitand_assign(&mut self, rhs: Self) {
+        self.0 &= !rhs.0
+    }
+}
+
 /// Option bit-flags for [`AST`] nodes.
 #[allow(non_snake_case)]
 pub mod AST_OPTION_FLAGS {
@@ -1016,12 +1058,20 @@ pub mod AST_OPTION_FLAGS {
     /// _(internals)_ The [`AST`][crate::AST] node is constant.
     /// Exported under the `internals` feature only.
     pub const AST_OPTION_CONSTANT: OptionFlags = OptionFlags(0b0000_0001);
-    /// _(internals)_ The [`AST`][crate::AST] node is exported.
+    /// _(internals)_ The [`AST`][crate::AST] node is public.
     /// Exported under the `internals` feature only.
-    pub const AST_OPTION_EXPORTED: OptionFlags = OptionFlags(0b0000_0010);
+    pub const AST_OPTION_PUBLIC: OptionFlags = OptionFlags(0b0000_0010);
     /// _(internals)_ The [`AST`][crate::AST] node is in negated mode.
     /// Exported under the `internals` feature only.
     pub const AST_OPTION_NEGATED: OptionFlags = OptionFlags(0b0000_0100);
+    /// _(internals)_ The [`AST`][crate::AST] node breaks out of normal control flow.
+    /// Exported under the `internals` feature only.
+    pub const AST_OPTION_BREAK_OUT: OptionFlags = OptionFlags(0b0000_1000);
+    /// _(internals)_ Mask of all options.
+    /// Exported under the `internals` feature only.
+    pub(crate) const AST_OPTION_ALL: OptionFlags = OptionFlags(
+        AST_OPTION_CONSTANT.0 | AST_OPTION_PUBLIC.0 | AST_OPTION_NEGATED.0 | AST_OPTION_BREAK_OUT.0,
+    );
 
     impl std::fmt::Debug for OptionFlags {
         fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
@@ -1046,8 +1096,9 @@ pub mod AST_OPTION_FLAGS {
 
             f.write_str("(")?;
             write_option(self, f, num_flags, AST_OPTION_CONSTANT, "Constant")?;
-            write_option(self, f, num_flags, AST_OPTION_EXPORTED, "Exported")?;
+            write_option(self, f, num_flags, AST_OPTION_PUBLIC, "Public")?;
             write_option(self, f, num_flags, AST_OPTION_NEGATED, "Negated")?;
+            write_option(self, f, num_flags, AST_OPTION_BREAK_OUT, "Break")?;
             f.write_str(")")?;
 
             Ok(())
@@ -1081,8 +1132,8 @@ pub enum Stmt {
     ///
     /// ### Option Flags
     ///
-    /// * [`AST_FLAG_NONE`][AST_FLAGS::AST_FLAG_NONE] = `while`
-    /// * [`AST_FLAG_NEGATED`][AST_FLAGS::AST_FLAG_NEGATED] = `until`
+    /// * [`AST_OPTION_NONE`][AST_OPTION_FLAGS::AST_OPTION_NONE] = `while`
+    /// * [`AST_OPTION_NEGATED`][AST_OPTION_FLAGS::AST_OPTION_NEGATED] = `until`
     Do(Box<StmtBlock>, Expr, OptionFlags, Position),
     /// `for` `(` id `,` counter `)` `in` expr `{` stmt `}`
     For(Expr, Box<(Ident, Option<Ident>, StmtBlock)>, Position),
@@ -1090,8 +1141,8 @@ pub enum Stmt {
     ///
     /// ### Option Flags
     ///
-    /// * [`AST_FLAG_EXPORTED`][AST_FLAGS::AST_FLAG_EXPORTED] = `export`
-    /// * [`AST_FLAG_CONSTANT`][AST_FLAGS::AST_FLAG_CONSTANT] = `const`
+    /// * [`AST_OPTION_PUBLIC`][AST_OPTION_FLAGS::AST_OPTION_PUBLIC] = `export`
+    /// * [`AST_OPTION_CONSTANT`][AST_OPTION_FLAGS::AST_OPTION_CONSTANT] = `const`
     Var(Expr, Box<Ident>, OptionFlags, Position),
     /// expr op`=` expr
     Assignment(Box<(Expr, Option<OpAssignment<'static>>, Expr)>, Position),
@@ -1106,12 +1157,20 @@ pub enum Stmt {
     TryCatch(Box<(StmtBlock, Option<Ident>, StmtBlock)>, Position),
     /// [expression][Expr]
     Expr(Expr),
-    /// `continue`
-    Continue(Position),
-    /// `break`
-    Break(Position),
+    /// `continue`/`break`
+    ///
+    /// ### Option Flags
+    ///
+    /// * [`AST_OPTION_NONE`][AST_OPTION_FLAGS::AST_OPTION_NONE] = `continue`
+    /// * [`AST_OPTION_BREAK_OUT`][AST_OPTION_FLAGS::AST_OPTION_BREAK_OUT] = `break`
+    BreakLoop(OptionFlags, Position),
     /// `return`/`throw`
-    Return(ReturnType, Option<Expr>, Position),
+    ///
+    /// ### Option Flags
+    ///
+    /// * [`AST_OPTION_NONE`][AST_OPTION_FLAGS::AST_OPTION_NONE] = `return`
+    /// * [`AST_OPTION_BREAK_OUT`][AST_OPTION_FLAGS::AST_OPTION_BREAK_OUT] = `throw`
+    Return(OptionFlags, Option<Expr>, Position),
     /// `import` expr `as` var
     ///
     /// Not available under `no_module`.
@@ -1125,6 +1184,11 @@ pub enum Stmt {
     /// Convert a variable to shared.
     ///
     /// Not available under `no_closure`.
+    ///
+    /// # Notes
+    ///
+    /// This variant does not map to any language structure.  It is currently only used only to
+    /// convert a normal variable into a shared variable when the variable is _captured_ by a closure.
     #[cfg(not(feature = "no_closure"))]
     Share(Identifier),
 }
@@ -1162,8 +1226,7 @@ impl Stmt {
     pub const fn position(&self) -> Position {
         match self {
             Self::Noop(pos)
-            | Self::Continue(pos)
-            | Self::Break(pos)
+            | Self::BreakLoop(_, pos)
             | Self::Block(_, pos)
             | Self::Assignment(_, pos)
             | Self::FnCall(_, pos)
@@ -1191,8 +1254,7 @@ impl Stmt {
     pub fn set_position(&mut self, new_pos: Position) -> &mut Self {
         match self {
             Self::Noop(pos)
-            | Self::Continue(pos)
-            | Self::Break(pos)
+            | Self::BreakLoop(_, pos)
             | Self::Block(_, pos)
             | Self::Assignment(_, pos)
             | Self::FnCall(_, pos)
@@ -1238,8 +1300,7 @@ impl Stmt {
 
             Self::Var(_, _, _, _)
             | Self::Assignment(_, _)
-            | Self::Continue(_)
-            | Self::Break(_)
+            | Self::BreakLoop(_, _)
             | Self::Return(_, _, _) => false,
 
             #[cfg(not(feature = "no_module"))]
@@ -1263,13 +1324,14 @@ impl Stmt {
             // A No-op requires a semicolon in order to know it is an empty statement!
             Self::Noop(_) => false,
 
+            Self::Expr(Expr::Custom(x, _)) if x.is_self_terminated() => true,
+
             Self::Var(_, _, _, _)
             | Self::Assignment(_, _)
-            | Self::FnCall(_, _)
             | Self::Expr(_)
+            | Self::FnCall(_, _)
             | Self::Do(_, _, _, _)
-            | Self::Continue(_)
-            | Self::Break(_)
+            | Self::BreakLoop(_, _)
             | Self::Return(_, _, _) => false,
 
             #[cfg(not(feature = "no_module"))]
@@ -1318,7 +1380,7 @@ impl Stmt {
 
             Self::Var(_, _, _, _) | Self::Assignment(_, _) | Self::FnCall(_, _) => false,
             Self::Block(block, _) => block.iter().all(|stmt| stmt.is_pure()),
-            Self::Continue(_) | Self::Break(_) | Self::Return(_, _, _) => false,
+            Self::BreakLoop(_, _) | Self::Return(_, _, _) => false,
             Self::TryCatch(x, _) => {
                 (x.0).0.iter().all(Stmt::is_pure) && (x.2).0.iter().all(Stmt::is_pure)
             }
@@ -1336,8 +1398,8 @@ impl Stmt {
     ///
     /// An internally pure statement only has side effects that disappear outside the block.
     ///
-    /// Only variable declarations (i.e. `let` and `const`) and `import`/`export` statements
-    /// are internally pure.
+    /// Currently only variable declarations (i.e. `let` and `const`) and `import`/`export`
+    /// statements are internally pure.
     #[inline]
     #[must_use]
     pub fn is_internally_pure(&self) -> bool {
@@ -1361,7 +1423,7 @@ impl Stmt {
     #[must_use]
     pub const fn is_control_flow_break(&self) -> bool {
         match self {
-            Self::Return(_, _, _) | Self::Break(_) | Self::Continue(_) => true,
+            Self::Return(_, _, _) | Self::BreakLoop(_, _) => true,
             _ => false,
         }
     }
@@ -1488,8 +1550,7 @@ impl Stmt {
             _ => (),
         }
 
-        path.pop()
-            .expect("never fails because `path` always contains the current node");
+        path.pop().expect("`path` contains current node");
 
         true
     }
@@ -1505,10 +1566,24 @@ impl Stmt {
 pub struct CustomExpr {
     /// List of keywords.
     pub keywords: StaticVec<Expr>,
-    /// Is the current [`Scope`][crate::Scope] modified?
+    /// Is the current [`Scope`][crate::Scope] possibly modified by this custom statement
+    /// (e.g. introducing a new variable)?
     pub scope_may_be_changed: bool,
     /// List of tokens actually parsed.
     pub tokens: StaticVec<Identifier>,
+    /// Is this custom syntax self-terminated?
+    pub self_terminated: bool,
+}
+
+impl CustomExpr {
+    /// Is this custom syntax self-terminated (i.e. no need for a semicolon terminator)?
+    ///
+    /// A self-terminated custom syntax always ends in `$block$`, `}` or `;`
+    #[must_use]
+    #[inline(always)]
+    pub const fn is_self_terminated(&self) -> bool {
+        self.self_terminated
+    }
 }
 
 /// _(internals)_ A binary expression.
@@ -1551,9 +1626,9 @@ impl OpAssignment<'_> {
     pub fn new(op: Token) -> Self {
         let op_raw = op
             .map_op_assignment()
-            .expect("never fails because token must be an op-assignment operator")
-            .keyword_syntax();
-        let op_assignment = op.keyword_syntax();
+            .expect("token is op-assignment operator")
+            .literal_syntax();
+        let op_assignment = op.literal_syntax();
 
         Self {
             hash_op_assign: calc_fn_hash(op_assignment, 2),
@@ -1665,6 +1740,9 @@ pub struct FnCallExpr {
     /// List of function call argument expressions.
     pub args: StaticVec<Expr>,
     /// List of function call arguments that are constants.
+    ///
+    /// Any arguments in `args` that is [`Expr::Stack`][Expr::Stack] indexes into this
+    /// array to find the constant for use as its argument value.
     pub constants: smallvec::SmallVec<[Dynamic; 2]>,
     /// Function name.
     pub name: Identifier,
@@ -1865,6 +1943,12 @@ pub enum Expr {
         )>,
     ),
     /// Stack slot
+    ///
+    /// # Notes
+    ///
+    /// This variant does not map to any language structure.  It is currently only used in function
+    /// calls with constant arguments where the `usize` number indexes into an array containing a
+    /// list of constant arguments for the function call.  See [`FnCallExpr`] for more details.
     Stack(usize, Position),
     /// { [statement][Stmt] ... }
     Stmt(Box<StmtBlock>),
@@ -2003,7 +2087,7 @@ impl Expr {
                 let mut arr = Array::with_capacity(x.len());
                 arr.extend(x.iter().map(|v| {
                     v.get_literal_value()
-                        .expect("never fails because a constant array always has a constant value")
+                        .expect("constant array has constant value")
                 }));
                 Dynamic::from_array(arr)
             }
@@ -2013,9 +2097,9 @@ impl Expr {
                 let mut map = x.1.clone();
                 x.0.iter().for_each(|(k, v)| {
                     *map.get_mut(k.name.as_str())
-                        .expect("never fails because the template should contain all the keys") = v
+                        .expect("template contains all keys") = v
                         .get_literal_value()
-                        .expect("never fails because a constant map always has a constant value")
+                        .expect("constant map has constant value")
                 });
                 Dynamic::from_map(map)
             }
@@ -2294,43 +2378,8 @@ impl Expr {
             _ => (),
         }
 
-        path.pop()
-            .expect("never fails because `path` always contains the current node");
+        path.pop().expect("`path` contains current node");
 
         true
     }
 }
-
-#[cfg(test)]
-mod tests {
-    /// This test is to make sure no code changes increase the sizes of critical data structures.
-    #[test]
-    fn check_struct_sizes() {
-        use crate::*;
-        use std::mem::size_of;
-
-        assert_eq!(size_of::<Dynamic>(), 16);
-        assert_eq!(size_of::<Option<Dynamic>>(), 16);
-        #[cfg(not(feature = "no_position"))]
-        assert_eq!(size_of::<Position>(), 4);
-        assert_eq!(size_of::<ast::Expr>(), 16);
-        assert_eq!(size_of::<Option<ast::Expr>>(), 16);
-        assert_eq!(size_of::<ast::Stmt>(), 32);
-        assert_eq!(size_of::<Option<ast::Stmt>>(), 32);
-        assert_eq!(
-            size_of::<FnPtr>(),
-            if cfg!(feature = "no_smartstring") {
-                80
-            } else {
-                96
-            }
-        );
-        assert_eq!(size_of::<Scope>(), 464);
-        assert_eq!(size_of::<LexError>(), 56);
-        assert_eq!(
-            size_of::<ParseError>(),
-            if cfg!(feature = "no_position") { 8 } else { 16 }
-        );
-        assert_eq!(size_of::<EvalAltResult>(), 72);
-    }
-}
