@@ -21,8 +21,26 @@ use crate::Array;
 #[cfg(not(feature = "no_object"))]
 use crate::Map;
 
-/// Engine public API
 impl Engine {
+    /// Get the global namespace module (which is the last module in `global_modules`).
+    #[inline(always)]
+    #[allow(dead_code)]
+    pub(crate) fn global_namespace(&self) -> &Module {
+        self.global_modules
+            .last()
+            .expect("global_modules contains at least one module")
+    }
+    /// Get a mutable reference to the global namespace module
+    /// (which is the last module in `global_modules`).
+    #[inline(always)]
+    pub(crate) fn global_namespace_mut(&mut self) -> &mut Module {
+        Shared::get_mut(
+            self.global_modules
+                .last_mut()
+                .expect("global_modules contains at least one module"),
+        )
+        .expect("global namespace module is never shared")
+    }
     /// Register a custom function with the [`Engine`].
     ///
     /// # Example
@@ -75,7 +93,7 @@ impl Engine {
         #[cfg(not(feature = "metadata"))]
         let param_type_names: Option<[&str; 0]> = None;
 
-        self.global_namespace.set_fn(
+        self.global_namespace_mut().set_fn(
             name,
             FnNamespace::Global,
             FnAccess::Public,
@@ -133,7 +151,7 @@ impl Engine {
         #[cfg(not(feature = "metadata"))]
         let param_type_names: Option<[&str; 0]> = None;
 
-        self.global_namespace.set_fn(
+        self.global_namespace_mut().set_fn(
             name,
             FnNamespace::Global,
             FnAccess::Public,
@@ -172,7 +190,7 @@ impl Engine {
         N: AsRef<str> + Into<Identifier>,
         T: Variant + Clone,
     {
-        self.global_namespace.set_raw_fn(
+        self.global_namespace_mut().set_raw_fn(
             name,
             FnNamespace::Global,
             FnAccess::Public,
@@ -184,8 +202,6 @@ impl Engine {
     /// Register a custom type for use with the [`Engine`].
     /// The type must implement [`Clone`].
     ///
-    /// Not available under `no_object`.
-    ///
     /// # Example
     ///
     /// ```
@@ -195,9 +211,8 @@ impl Engine {
     /// }
     ///
     /// impl TestStruct {
-    ///     fn new() -> Self                    { Self { field: 1 } }
-    ///
-    ///     fn update(&mut self, offset: i64)   { self.field += offset; }
+    ///     fn new() -> Self { Self { field: 1 } }
+    ///     fn update(&mut self, offset: i64) { self.field += offset; }
     /// }
     ///
     /// # fn main() -> Result<(), Box<rhai::EvalAltResult>> {
@@ -212,6 +227,7 @@ impl Engine {
     ///     // Use `register_fn` to register methods on the type.
     ///     .register_fn("update", TestStruct::update);
     ///
+    /// # #[cfg(not(feature = "no_object"))]
     /// assert_eq!(
     ///     engine.eval::<TestStruct>("let x = new_ts(); x.update(41); x")?,
     ///     TestStruct { field: 42 }
@@ -219,7 +235,6 @@ impl Engine {
     /// # Ok(())
     /// # }
     /// ```
-    #[cfg(not(feature = "no_object"))]
     #[inline(always)]
     pub fn register_type<T: Variant + Clone>(&mut self) -> &mut Self {
         self.register_type_with_name::<T>(type_name::<T>())
@@ -227,8 +242,6 @@ impl Engine {
     /// Register a custom type for use with the [`Engine`], with a pretty-print name
     /// for the `type_of` function. The type must implement [`Clone`].
     ///
-    /// Not available under `no_object`.
-    ///
     /// # Example
     ///
     /// ```
@@ -266,7 +279,6 @@ impl Engine {
     /// # Ok(())
     /// # }
     /// ```
-    #[cfg(not(feature = "no_object"))]
     #[inline(always)]
     pub fn register_type_with_name<T: Variant + Clone>(&mut self, name: &str) -> &mut Self {
         // Add the pretty-print type name into the map
@@ -282,7 +294,7 @@ impl Engine {
         T: Variant + Clone + IntoIterator,
         <T as IntoIterator>::Item: Variant + Clone,
     {
-        self.global_namespace.set_iterable::<T>();
+        self.global_namespace_mut().set_iterable::<T>();
         self
     }
     /// Register a getter function for a member of a registered type with the [`Engine`].
@@ -300,8 +312,7 @@ impl Engine {
     /// }
     ///
     /// impl TestStruct {
-    ///     fn new() -> Self                { Self { field: 1 } }
-    ///
+    ///     fn new() -> Self { Self { field: 1 } }
     ///     // Even a getter must start with `&mut self` and not `&self`.
     ///     fn get_field(&mut self) -> i64  { self.field }
     /// }
@@ -349,7 +360,6 @@ impl Engine {
     ///
     /// impl TestStruct {
     ///     fn new() -> Self { Self { field: 1 } }
-    ///
     ///     // Even a getter must start with `&mut self` and not `&self`.
     ///     fn get_field(&mut self) -> Result<i64, Box<EvalAltResult>> {
     ///         Ok(self.field)
@@ -392,9 +402,8 @@ impl Engine {
     /// }
     ///
     /// impl TestStruct {
-    ///     fn new() -> Self                        { Self { field: 1 } }
-    ///
-    ///     fn set_field(&mut self, new_val: i64)   { self.field = new_val; }
+    ///     fn new() -> Self { Self { field: 1 } }
+    ///     fn set_field(&mut self, new_val: i64) { self.field = new_val; }
     /// }
     ///
     /// # fn main() -> Result<(), Box<rhai::EvalAltResult>> {
@@ -442,7 +451,6 @@ impl Engine {
     ///
     /// impl TestStruct {
     ///     fn new() -> Self { Self { field: 1 } }
-    ///
     ///     fn set_field(&mut self, new_val: i64) -> Result<(), Box<EvalAltResult>> {
     ///         self.field = new_val;
     ///         Ok(())
@@ -492,12 +500,10 @@ impl Engine {
     /// }
     ///
     /// impl TestStruct {
-    ///     fn new() -> Self                        { Self { field: 1 } }
-    ///
+    ///     fn new() -> Self { Self { field: 1 } }
     ///     // Even a getter must start with `&mut self` and not `&self`.
-    ///     fn get_field(&mut self) -> i64          { self.field }
-    ///
-    ///     fn set_field(&mut self, new_val: i64)   { self.field = new_val; }
+    ///     fn get_field(&mut self) -> i64 { self.field }
+    ///     fn set_field(&mut self, new_val: i64) { self.field = new_val; }
     /// }
     ///
     /// # fn main() -> Result<(), Box<rhai::EvalAltResult>> {
@@ -549,7 +555,6 @@ impl Engine {
     ///
     /// impl TestStruct {
     ///     fn new() -> Self { Self { fields: vec![1, 2, 3, 4, 5] } }
-    ///
     ///     // Even a getter must start with `&mut self` and not `&self`.
     ///     fn get_field(&mut self, index: i64) -> i64 { self.fields[index as usize] }
     /// }
@@ -623,7 +628,6 @@ impl Engine {
     ///
     /// impl TestStruct {
     ///     fn new() -> Self { Self { fields: vec![1, 2, 3, 4, 5] } }
-    ///
     ///     // Even a getter must start with `&mut self` and not `&self`.
     ///     fn get_field(&mut self, index: i64) -> Result<i64, Box<EvalAltResult>> {
     ///         Ok(self.fields[index as usize])
@@ -697,7 +701,6 @@ impl Engine {
     ///
     /// impl TestStruct {
     ///     fn new() -> Self { Self { fields: vec![1, 2, 3, 4, 5] } }
-    ///
     ///     fn set_field(&mut self, index: i64, value: i64) { self.fields[index as usize] = value; }
     /// }
     ///
@@ -771,7 +774,6 @@ impl Engine {
     ///
     /// impl TestStruct {
     ///     fn new() -> Self { Self { fields: vec![1, 2, 3, 4, 5] } }
-    ///
     ///     fn set_field(&mut self, index: i64, value: i64) -> Result<(), Box<EvalAltResult>> {
     ///         self.fields[index as usize] = value;
     ///         Ok(())
@@ -847,11 +849,9 @@ impl Engine {
     /// }
     ///
     /// impl TestStruct {
-    ///     fn new() -> Self                                { Self { fields: vec![1, 2, 3, 4, 5] } }
-    ///
+    ///     fn new() -> Self { Self { fields: vec![1, 2, 3, 4, 5] } }
     ///     // Even a getter must start with `&mut self` and not `&self`.
-    ///     fn get_field(&mut self, index: i64) -> i64      { self.fields[index as usize] }
-    ///
+    ///     fn get_field(&mut self, index: i64) -> i64 { self.fields[index as usize] }
     ///     fn set_field(&mut self, index: i64, value: i64) { self.fields[index as usize] = value; }
     /// }
     ///
@@ -957,24 +957,18 @@ impl Engine {
                 }
             } else {
                 let mut iter = name.as_ref().splitn(2, separator.as_ref());
-                let sub_module = iter
-                    .next()
-                    .expect("never fails because the name contains a separator")
-                    .trim();
-                let remainder = iter
-                    .next()
-                    .expect("never fails because the name contains a separator")
-                    .trim();
+                let sub_module = iter.next().expect("name contains separator").trim();
+                let remainder = iter.next().expect("name contains separator").trim();
 
                 if !root.contains_key(sub_module) {
-                    let mut m: Module = Default::default();
+                    let mut m = Module::new();
                     register_static_module_raw(m.sub_modules_mut(), remainder, module);
                     m.build_index();
                     root.insert(sub_module.into(), m.into());
                 } else {
                     let m = root
                         .remove(sub_module)
-                        .expect("never fails because the root contains the sub-module");
+                        .expect("root contains the sub-module");
                     let mut m = crate::fn_native::shared_take_or_clone(m);
                     register_static_module_raw(m.sub_modules_mut(), remainder, module);
                     m.build_index();
@@ -1079,19 +1073,18 @@ impl Engine {
             resolver: &StaticModuleResolver,
             imports: &mut BTreeSet<Identifier>,
         ) {
-            ast.walk(&mut |path| match path
-                .last()
-                .expect("never fails because `path` always contains the current node")
-            {
-                // Collect all `import` statements with a string constant path
-                ASTNode::Stmt(Stmt::Import(Expr::StringConstant(s, _), _, _))
-                    if !resolver.contains_path(s) && !imports.contains(s.as_str()) =>
-                {
-                    imports.insert(s.clone().into());
-                    true
-                }
-                _ => true,
-            });
+            ast.walk(
+                &mut |path| match path.last().expect("`path` contains the current node") {
+                    // Collect all `import` statements with a string constant path
+                    ASTNode::Stmt(Stmt::Import(Expr::StringConstant(s, _), _, _))
+                        if !resolver.contains_path(s) && !imports.contains(s.as_str()) =>
+                    {
+                        imports.insert(s.clone().into());
+                        true
+                    }
+                    _ => true,
+                },
+            );
         }
 
         let mut ast = self.compile_scripts_with_scope(scope, &[script])?;
@@ -1188,7 +1181,8 @@ impl Engine {
         scripts: &[&str],
         optimization_level: OptimizationLevel,
     ) -> Result<AST, ParseError> {
-        let (stream, tokenizer_control) = self.lex_raw(scripts, None);
+        let (stream, tokenizer_control) =
+            self.lex_raw(scripts, self.token_mapper.as_ref().map(Box::as_ref));
         let mut state = ParseState::new(self, tokenizer_control);
         self.parse(
             &mut stream.peekable(),
@@ -1335,16 +1329,17 @@ impl Engine {
     /// true)?;
     ///
     /// assert_eq!(map.len(), 4);
-    /// assert_eq!(map["a"].as_int().unwrap(), 123);
-    /// assert_eq!(map["b"].as_int().unwrap(), 42);
+    /// assert_eq!(map["a"].as_int().expect("a should exist"), 123);
+    /// assert_eq!(map["b"].as_int().expect("b should exist"), 42);
     /// assert!(map["d"].is::<()>());
     ///
-    /// let c = map["c"].read_lock::<Map>().unwrap();
-    /// assert_eq!(c["x"].as_bool().unwrap(), false);
+    /// let c = map["c"].read_lock::<Map>().expect("c should exist");
+    /// assert_eq!(c["x"].as_bool().expect("x should be bool"), false);
     /// # Ok(())
     /// # }
     /// ```
     #[cfg(not(feature = "no_object"))]
+    #[inline(always)]
     pub fn parse_json(
         &self,
         json: impl AsRef<str>,
@@ -1352,52 +1347,53 @@ impl Engine {
     ) -> Result<Map, Box<EvalAltResult>> {
         use crate::token::Token;
 
-        let json = json.as_ref();
-        let mut scope = Default::default();
-
-        // Trims the JSON string and add a '#' in front
-        let json_text = json.trim_start();
-        let scripts = if json_text.starts_with(Token::MapStart.keyword_syntax()) {
-            [json_text, ""]
-        } else if json_text.starts_with(Token::LeftBrace.keyword_syntax()) {
-            ["#", json_text]
-        } else {
-            return Err(crate::ParseErrorType::MissingToken(
-                Token::LeftBrace.syntax().into(),
-                "to start a JSON object hash".into(),
-            )
-            .into_err(Position::new(1, (json.len() - json_text.len() + 1) as u16))
-            .into());
-        };
-
-        let (stream, tokenizer_control) = self.lex_raw(
-            &scripts,
-            Some(if has_null {
-                |token| match token {
-                    // If `null` is present, make sure `null` is treated as a variable
-                    Token::Reserved(s) if s == "null" => Token::Identifier(s),
-                    _ => token,
-                }
+        fn parse_json_inner(
+            engine: &Engine,
+            json: &str,
+            has_null: bool,
+        ) -> Result<Map, Box<EvalAltResult>> {
+            let mut scope = Scope::new();
+            let json_text = json.trim_start();
+            let scripts = if json_text.starts_with(Token::MapStart.literal_syntax()) {
+                [json_text, ""]
+            } else if json_text.starts_with(Token::LeftBrace.literal_syntax()) {
+                ["#", json_text]
             } else {
-                |t| t
-            }),
-        );
-
-        let mut state = ParseState::new(self, tokenizer_control);
-
-        let ast = self.parse_global_expr(
-            &mut stream.peekable(),
-            &mut state,
-            &scope,
-            OptimizationLevel::None,
-        )?;
-
-        // Handle null - map to ()
-        if has_null {
-            scope.push_constant("null", ());
+                return Err(crate::ParseErrorType::MissingToken(
+                    Token::LeftBrace.syntax().into(),
+                    "to start a JSON object hash".into(),
+                )
+                .into_err(Position::new(1, (json.len() - json_text.len() + 1) as u16))
+                .into());
+            };
+            let (stream, tokenizer_control) = engine.lex_raw(
+                &scripts,
+                if has_null {
+                    Some(&|token, _, _| {
+                        match token {
+                            // If `null` is present, make sure `null` is treated as a variable
+                            Token::Reserved(s) if s == "null" => Token::Identifier(s),
+                            _ => token,
+                        }
+                    })
+                } else {
+                    None
+                },
+            );
+            let mut state = ParseState::new(engine, tokenizer_control);
+            let ast = engine.parse_global_expr(
+                &mut stream.peekable(),
+                &mut state,
+                &scope,
+                OptimizationLevel::None,
+            )?;
+            if has_null {
+                scope.push_constant("null", ());
+            }
+            engine.eval_ast_with_scope(&mut scope, &ast)
         }
 
-        self.eval_ast_with_scope(&mut scope, &ast)
+        parse_json_inner(self, json.as_ref(), has_null)
     }
     /// Compile a string containing an expression into an [`AST`],
     /// which can be used later for evaluation.
@@ -1469,7 +1465,8 @@ impl Engine {
         script: &str,
     ) -> Result<AST, ParseError> {
         let scripts = [script];
-        let (stream, tokenizer_control) = self.lex_raw(&scripts, None);
+        let (stream, tokenizer_control) =
+            self.lex_raw(&scripts, self.token_mapper.as_ref().map(Box::as_ref));
 
         let mut peekable = stream.peekable();
         let mut state = ParseState::new(self, tokenizer_control);
@@ -1631,7 +1628,8 @@ impl Engine {
         script: &str,
     ) -> Result<T, Box<EvalAltResult>> {
         let scripts = [script];
-        let (stream, tokenizer_control) = self.lex_raw(&scripts, None);
+        let (stream, tokenizer_control) =
+            self.lex_raw(&scripts, self.token_mapper.as_ref().map(Box::as_ref));
         let mut state = ParseState::new(self, tokenizer_control);
 
         // No need to optimize a lone expression
@@ -1741,46 +1739,43 @@ impl Engine {
         let lib = &[ast.lib()];
         self.eval_global_statements(scope, mods, &mut state, statements, lib, level)
     }
-    /// Evaluate a file, but throw away the result and only return error (if any).
-    /// Useful for when you don't need the result, but still need to keep track of possible errors.
+    /// Evaluate a file, returning any error (if any).
     ///
     /// Not available under `no_std` or `WASM`.
     #[cfg(not(feature = "no_std"))]
     #[cfg(not(any(target_arch = "wasm32", target_arch = "wasm64")))]
     #[inline(always)]
-    pub fn consume_file(&self, path: std::path::PathBuf) -> Result<(), Box<EvalAltResult>> {
-        Self::read_file(path).and_then(|contents| self.consume(&contents))
+    pub fn run_file(&self, path: std::path::PathBuf) -> Result<(), Box<EvalAltResult>> {
+        Self::read_file(path).and_then(|contents| self.run(&contents))
     }
-    /// Evaluate a file with own scope, but throw away the result and only return error (if any).
-    /// Useful for when you don't need the result, but still need to keep track of possible errors.
+    /// Evaluate a file with own scope, returning any error (if any).
     ///
     /// Not available under `no_std` or `WASM`.
     #[cfg(not(feature = "no_std"))]
     #[cfg(not(any(target_arch = "wasm32", target_arch = "wasm64")))]
     #[inline(always)]
-    pub fn consume_file_with_scope(
+    pub fn run_file_with_scope(
         &self,
         scope: &mut Scope,
         path: std::path::PathBuf,
     ) -> Result<(), Box<EvalAltResult>> {
-        Self::read_file(path).and_then(|contents| self.consume_with_scope(scope, &contents))
+        Self::read_file(path).and_then(|contents| self.run_with_scope(scope, &contents))
     }
-    /// Evaluate a string, but throw away the result and only return error (if any).
-    /// Useful for when you don't need the result, but still need to keep track of possible errors.
+    /// Evaluate a script, returning any error (if any).
     #[inline(always)]
-    pub fn consume(&self, script: &str) -> Result<(), Box<EvalAltResult>> {
-        self.consume_with_scope(&mut Default::default(), script)
+    pub fn run(&self, script: &str) -> Result<(), Box<EvalAltResult>> {
+        self.run_with_scope(&mut Default::default(), script)
     }
-    /// Evaluate a string with own scope, but throw away the result and only return error (if any).
-    /// Useful for when you don't need the result, but still need to keep track of possible errors.
+    /// Evaluate a script with own scope, returning any error (if any).
     #[inline]
-    pub fn consume_with_scope(
+    pub fn run_with_scope(
         &self,
         scope: &mut Scope,
         script: &str,
     ) -> Result<(), Box<EvalAltResult>> {
         let scripts = [script];
-        let (stream, tokenizer_control) = self.lex_raw(&scripts, None);
+        let (stream, tokenizer_control) =
+            self.lex_raw(&scripts, self.token_mapper.as_ref().map(Box::as_ref));
         let mut state = ParseState::new(self, tokenizer_control);
 
         let ast = self.parse(
@@ -1790,18 +1785,16 @@ impl Engine {
             self.optimization_level,
         )?;
 
-        self.consume_ast_with_scope(scope, &ast)
+        self.run_ast_with_scope(scope, &ast)
     }
-    /// Evaluate an AST, but throw away the result and only return error (if any).
-    /// Useful for when you don't need the result, but still need to keep track of possible errors.
+    /// Evaluate an AST, returning any error (if any).
     #[inline(always)]
-    pub fn consume_ast(&self, ast: &AST) -> Result<(), Box<EvalAltResult>> {
-        self.consume_ast_with_scope(&mut Default::default(), ast)
+    pub fn run_ast(&self, ast: &AST) -> Result<(), Box<EvalAltResult>> {
+        self.run_ast_with_scope(&mut Default::default(), ast)
     }
-    /// Evaluate an [`AST`] with own scope, but throw away the result and only return error (if any).
-    /// Useful for when you don't need the result, but still need to keep track of possible errors.
+    /// Evaluate an [`AST`] with own scope, returning any error (if any).
     #[inline]
-    pub fn consume_ast_with_scope(
+    pub fn run_ast_with_scope(
         &self,
         scope: &mut Scope,
         ast: &AST,
@@ -1873,7 +1866,7 @@ impl Engine {
         name: impl AsRef<str>,
         args: impl crate::FuncArgs,
     ) -> Result<T, Box<EvalAltResult>> {
-        let mut arg_values: crate::StaticVec<_> = Default::default();
+        let mut arg_values = crate::StaticVec::new();
         args.parse(&mut arg_values);
         let mut args: crate::StaticVec<_> = arg_values.iter_mut().collect();
         let name = name.as_ref();
@@ -1939,7 +1932,7 @@ impl Engine {
     /// let mut value: Dynamic = 1_i64.into();
     /// let result = engine.call_fn_dynamic(&mut scope, &ast, true, "action", Some(&mut value), [ 41_i64.into() ])?;
     /// //                                                                    ^^^^^^^^^^^^^^^^ binding the 'this' pointer
-    /// assert_eq!(value.as_int().unwrap(), 42);
+    /// assert_eq!(value.as_int().expect("value should be INT"), 42);
     /// # }
     /// # Ok(())
     /// # }
@@ -1985,7 +1978,10 @@ impl Engine {
         let statements = ast.statements();
 
         if eval_ast && !statements.is_empty() {
+            // Make sure new variables introduced at global level do not _spill_ into the function call
+            let orig_scope_len = scope.len();
             self.eval_global_statements(scope, mods, state, statements, lib, 0)?;
+            scope.rewind(orig_scope_len);
         }
 
         let fn_def = ast
@@ -2041,7 +2037,7 @@ impl Engine {
             .map(|f| {
                 f.func
                     .get_script_fn_def()
-                    .expect("never fails because the function is scripted")
+                    .expect("scripted function")
                     .clone()
             })
             .collect();
@@ -2050,7 +2046,7 @@ impl Engine {
         let lib = Default::default();
 
         let stmt = std::mem::take(ast.statements_mut());
-        crate::optimize::optimize_into_ast(self, scope, stmt.into_vec(), lib, optimization_level)
+        crate::optimize::optimize_into_ast(self, scope, stmt, lib, optimization_level)
     }
     /// _(metadata)_ Generate a list of all registered functions.
     /// Exported under the `metadata` feature only.
@@ -2063,9 +2059,9 @@ impl Engine {
     #[inline]
     #[must_use]
     pub fn gen_fn_signatures(&self, include_packages: bool) -> Vec<String> {
-        let mut signatures: Vec<_> = Default::default();
+        let mut signatures = Vec::with_capacity(64);
 
-        signatures.extend(self.global_namespace.gen_fn_signatures());
+        signatures.extend(self.global_namespace().gen_fn_signatures());
 
         self.global_sub_modules.iter().for_each(|(name, m)| {
             signatures.extend(m.gen_fn_signatures().map(|f| format!("{}::{}", name, f)))
@@ -2075,6 +2071,7 @@ impl Engine {
             signatures.extend(
                 self.global_modules
                     .iter()
+                    .take(self.global_modules.len() - 1)
                     .flat_map(|m| m.gen_fn_signatures()),
             );
         }
@@ -2083,12 +2080,28 @@ impl Engine {
     }
     /// Provide a callback that will be invoked before each variable access.
     ///
-    /// # Return Value of Callback
+    /// # Callback Function Signature
     ///
-    /// Return `Ok(None)` to continue with normal variable access.  
-    /// Return `Ok(Some(Dynamic))` as the variable's value.
+    /// The callback function signature takes the following form:
     ///
-    /// # Errors in Callback
+    /// > `Fn(name: &str, index: usize, context: &EvalContext)`  
+    /// > `   -> Result<Option<Dynamic>, Box<EvalAltResult>> + 'static`
+    ///
+    /// where:
+    /// * `index`: an offset from the bottom of the current [`Scope`] that the variable is supposed
+    ///   to reside. Offsets start from 1, with 1 meaning the last variable in the current
+    ///   [`Scope`].  Essentially the correct variable is at position `scope.len() - index`.
+    ///   If `index` is zero, then there is no pre-calculated offset position and a search through the
+    ///   current [`Scope`] must be performed.
+    ///
+    /// * `context`: the current [evaluation context][`EvalContext`].
+    ///
+    /// ## Return value
+    ///
+    /// * `Ok(None)`: continue with normal variable access.
+    /// * `Ok(Some(Dynamic))`: the variable's value.
+    ///
+    /// ## Raising errors
     ///
     /// Return `Err(...)` if there is an error.
     ///
@@ -2123,10 +2136,81 @@ impl Engine {
         self.resolve_var = Some(Box::new(callback));
         self
     }
+    /// _(internals)_ Provide a callback that will be invoked during parsing to remap certain tokens.
+    /// Exported under the `internals` feature only.
+    ///
+    /// # Callback Function Signature
+    ///
+    /// The callback function signature takes the following form:
+    ///
+    /// > `Fn(token: Token, pos: Position, state: &TokenizeState) -> Token`
+    ///
+    /// where:
+    /// * [`token`][crate::token::Token]: current token parsed
+    /// * [`pos`][`Position`]: location of the token
+    /// * [`state`][crate::token::TokenizeState]: current state of the tokenizer
+    ///
+    /// ## Raising errors
+    ///
+    /// It is possible to raise a parsing error by returning
+    /// [`Token::LexError`][crate::token::Token::LexError] as the mapped token.
+    ///
+    /// # Example
+    ///
+    /// ```
+    /// # fn main() -> Result<(), Box<rhai::EvalAltResult>> {
+    /// use rhai::{Engine, Token};
+    ///
+    /// let mut engine = Engine::new();
+    ///
+    /// // Register a token mapper.
+    /// engine.on_parse_token(|token, _, _| {
+    ///     match token {
+    ///         // Convert all integer literals to strings
+    ///         Token::IntegerConstant(n) => Token::StringConstant(n.to_string()),
+    ///         // Convert 'begin' .. 'end' to '{' .. '}'
+    ///         Token::Identifier(s) if &s == "begin" => Token::LeftBrace,
+    ///         Token::Identifier(s) if &s == "end" => Token::RightBrace,
+    ///         // Pass through all other tokens unchanged
+    ///         _ => token
+    ///     }
+    /// });
+    ///
+    /// assert_eq!(engine.eval::<String>("42")?, "42");
+    /// assert_eq!(engine.eval::<bool>("true")?, true);
+    /// assert_eq!(engine.eval::<String>("let x = 42; begin let x = 0; end; x")?, "42");
+    ///
+    /// # Ok(())
+    /// # }
+    /// ```
+    #[cfg(feature = "internals")]
+    #[inline(always)]
+    pub fn on_parse_token(
+        &mut self,
+        callback: impl Fn(crate::token::Token, Position, &crate::token::TokenizeState) -> crate::token::Token
+            + SendSync
+            + 'static,
+    ) -> &mut Self {
+        use std::string::ParseError;
+
+        self.token_mapper = Some(Box::new(callback));
+        self
+    }
     /// Register a callback for script evaluation progress.
     ///
     /// Not available under `unchecked`.
     ///
+    /// # Callback Function Signature
+    ///
+    /// The callback function signature takes the following form:
+    ///
+    /// > `Fn(counter: u64) -> Option<Dynamic>`
+    ///
+    /// ## Return value
+    ///
+    /// * `None`: continue running the script.
+    /// * `Some(Dynamic)`: terminate the script with the specified exception value.
+    ///
     /// # Example
     ///
     /// ```
@@ -2151,8 +2235,8 @@ impl Engine {
     ///     }
     /// });
     ///
-    /// engine.consume("for x in range(0, 50000) {}")
-    ///     .expect_err("should error");
+    /// engine.run("for x in range(0, 50000) {}")
+    ///       .expect_err("should error");
     ///
     /// assert_eq!(*result.read().unwrap(), 9600);
     ///
@@ -2186,7 +2270,7 @@ impl Engine {
     /// let logger = result.clone();
     /// engine.on_print(move |s| logger.write().unwrap().push_str(s));
     ///
-    /// engine.consume("print(40 + 2);")?;
+    /// engine.run("print(40 + 2);")?;
     ///
     /// assert_eq!(*result.read().unwrap(), "42");
     /// # Ok(())
@@ -2199,6 +2283,17 @@ impl Engine {
     }
     /// Override default action of `debug` (print to stdout using [`println!`])
     ///
+    /// # Callback Function Signature
+    ///
+    /// The callback function signature passed takes the following form:
+    ///
+    /// > `Fn(text: &str, source: Option<&str>, pos: Position)`
+    ///
+    /// where:
+    /// * `text`: the text to display
+    /// * `source`: current source, if any
+    /// * [`pos`][`Position`]: location of the `debug` call
+    ///
     /// # Example
     ///
     /// ```
@@ -2219,7 +2314,7 @@ impl Engine {
     ///
     /// let mut ast = engine.compile(r#"let x = "hello"; debug(x);"#)?;
     /// ast.set_source("world");
-    /// engine.consume_ast(&ast)?;
+    /// engine.run_ast(&ast)?;
     ///
     /// #[cfg(not(feature = "no_position"))]
     /// assert_eq!(*result.read().unwrap(), r#"world @ 1:18 > "hello""#);
