@@ -279,7 +279,7 @@ impl Engine {
     ///
     /// // Register a custom operator called 'foo' and give it
     /// // a precedence of 160 (i.e. between +|- and *|/).
-    /// engine.register_custom_operator("foo", 160).unwrap();
+    /// engine.register_custom_operator("foo", 160).expect("should succeed");
     ///
     /// // Register a binary function named 'foo'
     /// engine.register_fn("foo", |x: i64, y: i64| (x * y) - (x + y));
@@ -307,13 +307,13 @@ impl Engine {
             None | Some(Token::Reserved(_)) | Some(Token::Custom(_)) => (),
             // Active standard keywords cannot be made custom
             // Disabled keywords are OK
-            Some(token) if token.is_keyword() => {
+            Some(token) if token.is_standard_keyword() => {
                 if !self.disabled_symbols.contains(token.syntax().as_ref()) {
                     return Err(format!("'{}' is a reserved keyword", keyword.as_ref()));
                 }
             }
             // Active standard symbols cannot be made custom
-            Some(token) if token.is_symbol() => {
+            Some(token) if token.is_standard_symbol() => {
                 if !self.disabled_symbols.contains(token.syntax().as_ref()) {
                     return Err(format!("'{}' is a reserved operator", keyword.as_ref()));
                 }
