@@ -190,17 +190,39 @@ impl Engine {
                 let mut bitmask = 1usize; // Bitmask of which parameter to replace with `Dynamic`
 
                 loop {
-                    let func = lib.iter().find_map(|m| m.get_fn(hash).cloned().map(|func| FnResolutionCacheEntry {
-                        func, source: m.id_raw().cloned()
-                    })).or_else(|| self.global_namespace.get_fn(hash).cloned().map(|func| FnResolutionCacheEntry {
-                        func, source: None
-                    })).or_else(|| self.global_modules.iter().find_map(|m| m.get_fn(hash).cloned().map(|func| FnResolutionCacheEntry {
-                        func, source: m.id_raw().cloned()
-                    }))).or_else(|| mods.get_fn(hash).map(|(func, source)| FnResolutionCacheEntry {
-                        func: func.clone(), source: source.cloned()
-                    })).or_else(|| self.global_sub_modules.values().find_map(|m| m.get_qualified_fn(hash).cloned().map(|func| FnResolutionCacheEntry {
-                        func, source: m.id_raw().cloned()
-                    })));
+                    let func = lib
+                        .iter()
+                        .find_map(|m| {
+                            m.get_fn(hash).cloned().map(|func| FnResolutionCacheEntry {
+                                func,
+                                source: m.id_raw().cloned(),
+                            })
+                        })
+                        .or_else(|| {
+                            self.global_modules.iter().find_map(|m| {
+                                m.get_fn(hash).cloned().map(|func| FnResolutionCacheEntry {
+                                    func,
+                                    source: m.id_raw().cloned(),
+                                })
+                            })
+                        })
+                        .or_else(|| {
+                            mods.get_fn(hash)
+                                .map(|(func, source)| FnResolutionCacheEntry {
+                                    func: func.clone(),
+                                    source: source.cloned(),
+                                })
+                        })
+                        .or_else(|| {
+                            self.global_sub_modules.values().find_map(|m| {
+                                m.get_qualified_fn(hash).cloned().map(|func| {
+                                    FnResolutionCacheEntry {
+                                        func,
+                                        source: m.id_raw().cloned(),
+                                    }
+                                })
+                            })
+                        });
 
                     match func {
                         // Specific version found
@@ -215,17 +237,26 @@ impl Engine {
                             return args.and_then(|args| {
                                 if !is_op_assignment {
                                     get_builtin_binary_op_fn(fn_name, &args[0], &args[1]).map(|f| {
-                                        FnResolutionCacheEntry { func: CallableFunction::from_method(
-                                            Box::new(f) as Box<FnAny>
-                                        ), source: None }
+                                        FnResolutionCacheEntry {
+                                            func: CallableFunction::from_method(
+                                                Box::new(f) as Box<FnAny>
+                                            ),
+                                            source: None,
+                                        }
                                     })
                                 } else {
-                                    let (first, second) = args.split_first()
-                                                              .expect("never fails because an op-assignment must have two arguments");
+                                    let (first, second) = args
+                                        .split_first()
+                                        .expect("op-assignment has two arguments");
 
-                                    get_builtin_op_assignment_fn(fn_name, *first, second[0]).map(|f| FnResolutionCacheEntry {
-                                        func: CallableFunction::from_method(Box::new(f) as Box<FnAny>), source: None
-                                    })
+                                    get_builtin_op_assignment_fn(fn_name, *first, second[0]).map(
+                                        |f| FnResolutionCacheEntry {
+                                            func: CallableFunction::from_method(
+                                                Box::new(f) as Box<FnAny>
+                                            ),
+                                            source: None,
+                                        },
+                                    )
                                 }
                                 .map(Box::new)
                             });
@@ -234,17 +265,19 @@ impl Engine {
                         // Try all permutations with `Dynamic` wildcards
                         None => {
                             let hash_params = calc_fn_params_hash(
-                                args.as_ref().expect("never fails because there are no permutations if there are no arguments")
-                                        .iter().enumerate().map(|(i, a)|
-                                {
-                                    let mask = 1usize << (num_args - i - 1);
-                                    if bitmask & mask != 0 {
-                                        // Replace with `Dynamic`
-                                        TypeId::of::<Dynamic>()
-                                    } else {
-                                        a.type_id()
-                                    }
-                                }),
+                                args.as_ref()
+                                    .expect("no permutations if no arguments")
+                                    .iter()
+                                    .enumerate()
+                                    .map(|(i, a)| {
+                                        let mask = 1usize << (num_args - i - 1);
+                                        if bitmask & mask != 0 {
+                                            // Replace with `Dynamic`
+                                            TypeId::of::<Dynamic>()
+                                        } else {
+                                            a.type_id()
+                                        }
+                                    }),
                             );
                             hash = combine_hashes(hash_script, hash_params);
 
@@ -290,7 +323,10 @@ impl Engine {
             let mut backup: Option<ArgBackup> = None;
             if is_method_call && func.is_pure() && !args.is_empty() {
                 backup = Some(Default::default());
-                backup.as_mut().unwrap().change_first_arg_to_copy(args);
+                backup
+                    .as_mut()
+                    .expect("`backup` is `Some`")
+                    .change_first_arg_to_copy(args);
             }
 
             // Run external function
@@ -301,12 +337,10 @@ impl Engine {
 
             let result = if func.is_plugin_fn() {
                 func.get_plugin_fn()
-                    .expect("never fails because the function is a plugin")
+                    .expect("plugin function")
                     .call((self, name, source, mods, lib).into(), args)
             } else {
-                let func = func
-                    .get_native_fn()
-                    .expect("never fails because the function is native");
+                let func = func.get_native_fn().expect("native function");
                 func((self, name, source, mods, lib).into(), args)
             };
 
@@ -321,7 +355,7 @@ impl Engine {
             return Ok(match name {
                 KEYWORD_PRINT => {
                     if let Some(ref print) = self.print {
-                        let text = result.as_immutable_string().map_err(|typ| {
+                        let text = result.into_immutable_string().map_err(|typ| {
                             EvalAltResult::ErrorMismatchOutputType(
                                 self.map_type_name(type_name::<ImmutableString>()).into(),
                                 typ.into(),
@@ -335,7 +369,7 @@ impl Engine {
                 }
                 KEYWORD_DEBUG => {
                     if let Some(ref debug) = self.debug {
-                        let text = result.as_immutable_string().map_err(|typ| {
+                        let text = result.into_immutable_string().map_err(|typ| {
                             EvalAltResult::ErrorMismatchOutputType(
                                 self.map_type_name(type_name::<ImmutableString>()).into(),
                                 typ.into(),
@@ -579,8 +613,6 @@ impl Engine {
 
         // First check script-defined functions
         let result = lib.iter().any(|&m| m.contains_fn(hash_script))
-            // Then check registered functions
-            || self.global_namespace.contains_fn(hash_script)
             // Then check packages
             || self.global_modules.iter().any(|m| m.contains_fn(hash_script))
             // Then check imported modules
@@ -644,19 +676,17 @@ impl Engine {
             {
                 let fn_name = args[0]
                     .read_lock::<ImmutableString>()
-                    .expect("never fails because `args[0]` is `FnPtr`");
-                let num_params = args[1]
-                    .as_int()
-                    .expect("never fails because `args[1]` is `INT`");
+                    .expect("`args[0]` is `FnPtr`");
+                let num_params = args[1].as_int().expect("`args[1]` is `INT`");
 
                 return Ok((
                     if num_params < 0 {
-                        Dynamic::FALSE
+                        false
                     } else {
                         let hash_script = calc_fn_hash(fn_name.as_str(), num_params as usize);
                         self.has_script_fn(Some(mods), state, lib, hash_script)
-                            .into()
-                    },
+                    }
+                    .into(),
                     false,
                 ));
             }
@@ -691,9 +721,7 @@ impl Engine {
             // Script function call
             assert!(func.is_script());
 
-            let func = func
-                .get_script_fn_def()
-                .expect("never fails because the function is scripted");
+            let func = func.get_script_fn_def().expect("scripted function");
 
             if func.body.is_empty() {
                 return Ok((Dynamic::UNIT, false));
@@ -719,7 +747,7 @@ impl Engine {
                 // Method call of script function - map first argument to `this`
                 let (first, rest) = args
                     .split_first_mut()
-                    .expect("never fails because a method call always has a first parameter");
+                    .expect("method call has first parameter");
 
                 let orig_source = state.source.take();
                 state.source = source;
@@ -748,7 +776,10 @@ impl Engine {
                 let mut backup: Option<ArgBackup> = None;
                 if is_ref_mut && !args.is_empty() {
                     backup = Some(Default::default());
-                    backup.as_mut().unwrap().change_first_arg_to_copy(args);
+                    backup
+                        .as_mut()
+                        .expect("`backup` is `Some`")
+                        .change_first_arg_to_copy(args);
                 }
 
                 let orig_source = state.source.take();
@@ -871,9 +902,7 @@ impl Engine {
         let (result, updated) = match fn_name {
             KEYWORD_FN_PTR_CALL if target.is::<FnPtr>() => {
                 // FnPtr call
-                let fn_ptr = target
-                    .read_lock::<FnPtr>()
-                    .expect("never fails because `obj` is `FnPtr`");
+                let fn_ptr = target.read_lock::<FnPtr>().expect("`obj` is `FnPtr`");
                 // Redirect function name
                 let fn_name = fn_ptr.fn_name();
                 let args_len = call_args.len() + fn_ptr.curry().len();
@@ -938,9 +967,7 @@ impl Engine {
                     ));
                 }
 
-                let fn_ptr = target
-                    .read_lock::<FnPtr>()
-                    .expect("never fails because `obj` is `FnPtr`");
+                let fn_ptr = target.read_lock::<FnPtr>().expect("`obj` is `FnPtr`");
 
                 // Curry call
                 Ok((
@@ -1100,7 +1127,7 @@ impl Engine {
 
                 // Fn - only in function call style
                 return arg
-                    .as_immutable_string()
+                    .into_immutable_string()
                     .map_err(|typ| self.make_type_mismatch_err::<ImmutableString>(typ, arg_pos))
                     .and_then(FnPtr::try_from)
                     .map(Into::<Dynamic>::into)
@@ -1150,7 +1177,7 @@ impl Engine {
                 )?;
 
                 let fn_name = arg
-                    .as_immutable_string()
+                    .into_immutable_string()
                     .map_err(|typ| self.make_type_mismatch_err::<ImmutableString>(typ, arg_pos))?;
 
                 let (arg, arg_pos) = self.get_arg_value(
@@ -1162,12 +1189,12 @@ impl Engine {
                     .map_err(|typ| self.make_type_mismatch_err::<crate::INT>(typ, arg_pos))?;
 
                 return Ok(if num_params < 0 {
-                    Dynamic::FALSE
+                    false
                 } else {
                     let hash_script = calc_fn_hash(&fn_name, num_params as usize);
                     self.has_script_fn(Some(mods), state, lib, hash_script)
-                        .into()
-                });
+                }
+                .into());
             }
 
             // Handle is_def_var()
@@ -1176,7 +1203,7 @@ impl Engine {
                     scope, mods, state, lib, this_ptr, level, args_expr, constants, 0,
                 )?;
                 let var_name = arg
-                    .as_immutable_string()
+                    .into_immutable_string()
                     .map_err(|typ| self.make_type_mismatch_err::<ImmutableString>(typ, arg_pos))?;
                 return Ok(scope.contains(&var_name).into());
             }
@@ -1189,7 +1216,7 @@ impl Engine {
                     scope, mods, state, lib, this_ptr, level, args_expr, constants, 0,
                 )?;
                 let script = &value
-                    .as_immutable_string()
+                    .into_immutable_string()
                     .map_err(|typ| self.make_type_mismatch_err::<ImmutableString>(typ, pos))?;
                 let result =
                     self.eval_script_expr_in_place(scope, mods, state, lib, script, pos, level + 1);
@@ -1264,7 +1291,7 @@ impl Engine {
                 } else {
                     // Turn it into a method call only if the object is not shared and not a simple value
                     is_ref_mut = true;
-                    let obj_ref = target.take_ref().expect("never fails because `target` is a reference if it is not a value and not shared");
+                    let obj_ref = target.take_ref().expect("`target` is reference");
                     args.push(obj_ref);
                     args.extend(arg_values.iter_mut());
                 }
@@ -1345,9 +1372,9 @@ impl Engine {
                     // Turn it into a method call only if the object is not shared and not a simple value
                     let (first, rest) = arg_values
                         .split_first_mut()
-                        .expect("never fails because the arguments list is not empty");
+                        .expect("arguments list is not empty");
                     first_arg_value = Some(first);
-                    let obj_ref = target.take_ref().expect("never fails because `target` is a reference if it is not a value and not shared");
+                    let obj_ref = target.take_ref().expect("`target` is reference");
                     args.push(obj_ref);
                     args.extend(rest.iter_mut());
                 }
@@ -1393,9 +1420,7 @@ impl Engine {
         match func {
             #[cfg(not(feature = "no_function"))]
             Some(f) if f.is_script() => {
-                let fn_def = f
-                    .get_script_fn_def()
-                    .expect("never fails because the function is scripted");
+                let fn_def = f.get_script_fn_def().expect("scripted function");
 
                 if fn_def.body.is_empty() {
                     Ok(Dynamic::UNIT)
@@ -1419,15 +1444,13 @@ impl Engine {
 
             Some(f) if f.is_plugin_fn() => f
                 .get_plugin_fn()
-                .expect("never fails because the function is a plugin")
+                .expect("plugin function")
                 .clone()
                 .call((self, fn_name, module.id(), &*mods, lib).into(), &mut args)
                 .map_err(|err| err.fill_position(pos)),
 
             Some(f) if f.is_native() => {
-                let func = f
-                    .get_native_fn()
-                    .expect("never fails because the function is native");
+                let func = f.get_native_fn().expect("native function");
                 func((self, fn_name, module.id(), &*mods, lib).into(), &mut args)
                     .map_err(|err| err.fill_position(pos))
             }
