@@ -4,6 +4,7 @@ use crate::engine::{
     Precedence, KEYWORD_DEBUG, KEYWORD_EVAL, KEYWORD_FN_PTR, KEYWORD_FN_PTR_CALL,
     KEYWORD_FN_PTR_CURRY, KEYWORD_IS_DEF_VAR, KEYWORD_PRINT, KEYWORD_THIS, KEYWORD_TYPE_OF,
 };
+use crate::fn_native::OnParseTokenCallback;
 use crate::{Engine, LexError, StaticVec, INT};
 #[cfg(feature = "no_std")]
 use std::prelude::v1::*;
@@ -28,6 +29,10 @@ use rust_decimal::Decimal;
 use crate::engine::KEYWORD_IS_DEF_FN;
 
 /// _(internals)_ A type containing commands to control the tokenizer.
+///
+/// # Volatile Data Structure
+///
+/// This type is volatile and may change.
 #[derive(Debug, Clone, Eq, PartialEq, Hash, Copy, Default)]
 pub struct TokenizerControlBlock {
     /// Is the current tokenizer position within an interpolated text string?
@@ -483,9 +488,9 @@ pub enum Token {
 }
 
 impl Token {
-    /// Get the syntax of the token if it is a keyword.
+    /// Get the literal syntax of the token.
     #[must_use]
-    pub const fn keyword_syntax(&self) -> &'static str {
+    pub const fn literal_syntax(&self) -> &'static str {
         use Token::*;
 
         match self {
@@ -595,7 +600,7 @@ impl Token {
 
             EOF => "{EOF}".into(),
 
-            token => token.keyword_syntax().into(),
+            token => token.literal_syntax().into(),
         }
     }
 
@@ -912,7 +917,7 @@ impl Token {
 
     /// Is this token a standard symbol used in the language?
     #[must_use]
-    pub const fn is_symbol(&self) -> bool {
+    pub const fn is_standard_symbol(&self) -> bool {
         use Token::*;
 
         match self {
@@ -928,10 +933,10 @@ impl Token {
         }
     }
 
-    /// Is this token an active standard keyword?
+    /// Is this token a standard keyword?
     #[inline]
     #[must_use]
-    pub const fn is_keyword(&self) -> bool {
+    pub const fn is_standard_keyword(&self) -> bool {
         use Token::*;
 
         match self {
@@ -948,7 +953,7 @@ impl Token {
         }
     }
 
-    /// Is this token a reserved symbol?
+    /// Is this token a reserved keyword or symbol?
     #[inline(always)]
     #[must_use]
     pub const fn is_reserved(&self) -> bool {
@@ -960,7 +965,7 @@ impl Token {
     #[inline]
     pub(crate) fn into_function_name_for_override(self) -> Result<String, Self> {
         match self {
-            Self::Custom(s) | Self::Identifier(s) if is_valid_identifier(s.chars()) => Ok(s),
+            Self::Custom(s) | Self::Identifier(s) if is_valid_function_name(&s) => Ok(s),
             _ => Err(self),
         }
     }
@@ -991,7 +996,7 @@ pub struct TokenizeState {
     /// Maximum length of a string.
     pub max_string_size: Option<NonZeroUsize>,
     /// Can the next token be a unary operator?
-    pub non_unary: bool,
+    pub next_token_cannot_be_unary: bool,
     /// Is the tokenizer currently inside a block comment?
     pub comment_level: usize,
     /// Include comments?
@@ -1207,7 +1212,7 @@ pub fn parse_string_literal(
                 {
                     let start_position = start
                         .position()
-                        .expect("never fails because the string must have a starting position");
+                        .expect("string must have starting position");
                     skip_whitespace_until = start_position + 1;
                 }
             }
@@ -1229,9 +1234,7 @@ pub fn parse_string_literal(
             // Whitespace to skip
             #[cfg(not(feature = "no_position"))]
             _ if next_char.is_whitespace()
-                && pos
-                    .position()
-                    .expect("never fails because a character must have a position")
+                && pos.position().expect("character must have position")
                     < skip_whitespace_until => {}
 
             // All other characters
@@ -1328,7 +1331,7 @@ pub fn get_next_token(
 
     // Save the last token's state
     if let Some((ref token, _)) = result {
-        state.non_unary = !token.is_next_unary();
+        state.next_token_cannot_be_unary = !token.is_next_unary();
     }
 
     result
@@ -1379,16 +1382,12 @@ fn get_next_token_inner(
 
         #[cfg(not(feature = "no_function"))]
         #[cfg(feature = "metadata")]
-        let return_comment = return_comment
-            || is_doc_comment(
-                comment
-                    .as_ref()
-                    .expect("never fails because `include_comments` is true"),
-            );
+        let return_comment =
+            return_comment || is_doc_comment(comment.as_ref().expect("`include_comments` is true"));
 
         if return_comment {
             return Some((
-                Token::Comment(comment.expect("never fails because `return_comment` is true")),
+                Token::Comment(comment.expect("`return_comment` is true")),
                 start_pos,
             ));
         }
@@ -1427,7 +1426,7 @@ fn get_next_token_inner(
 
             // digit ...
             ('0'..='9', _) => {
-                let mut result: smallvec::SmallVec<[char; 16]> = Default::default();
+                let mut result = smallvec::SmallVec::<[char; 16]>::new();
                 let mut radix_base: Option<u32> = None;
                 let mut valid: fn(char) -> bool = is_numeric_digit;
                 result.push(c);
@@ -1440,7 +1439,7 @@ fn get_next_token_inner(
                         }
                         #[cfg(any(not(feature = "no_float"), feature = "decimal"))]
                         '.' => {
-                            stream.get_next().expect("never fails because it is `.`");
+                            stream.get_next().expect("it is `.`");
 
                             // Check if followed by digits or something that cannot start a property name
                             match stream.peek_next().unwrap_or('\0') {
@@ -1474,7 +1473,7 @@ fn get_next_token_inner(
                         }
                         #[cfg(not(feature = "no_float"))]
                         'e' => {
-                            stream.get_next().expect("never fails it is `e`");
+                            stream.get_next().expect("it is `e`");
 
                             // Check if followed by digits or +/-
                             match stream.peek_next().unwrap_or('\0') {
@@ -1487,11 +1486,7 @@ fn get_next_token_inner(
                                 '+' | '-' => {
                                     result.push(next_char);
                                     pos.advance();
-                                    result.push(
-                                        stream
-                                            .get_next()
-                                            .expect("never fails because it is `+` or `-`"),
-                                    );
+                                    result.push(stream.get_next().expect("it is `+` or `-`"));
                                     pos.advance();
                                 }
                                 // Not a floating-point number
@@ -1639,7 +1634,7 @@ fn get_next_token_inner(
                         |(err, err_pos)| (Token::LexError(err), err_pos),
                         |(result, _)| {
                             let mut chars = result.chars();
-                            let first = chars.next().unwrap();
+                            let first = chars.next().expect("`chars` is not empty");
 
                             if chars.next().is_some() {
                                 (Token::LexError(LERR::MalformedChar(result)), start_pos)
@@ -1687,10 +1682,12 @@ fn get_next_token_inner(
                 eat_next(stream, pos);
                 return Some((Token::Reserved("++".into()), start_pos));
             }
-            ('+', _) if !state.non_unary => return Some((Token::UnaryPlus, start_pos)),
+            ('+', _) if !state.next_token_cannot_be_unary => {
+                return Some((Token::UnaryPlus, start_pos))
+            }
             ('+', _) => return Some((Token::Plus, start_pos)),
 
-            ('-', '0'..='9') if !state.non_unary => negated = Some(start_pos),
+            ('-', '0'..='9') if !state.next_token_cannot_be_unary => negated = Some(start_pos),
             ('-', '0'..='9') => return Some((Token::Minus, start_pos)),
             ('-', '=') => {
                 eat_next(stream, pos);
@@ -1704,7 +1701,9 @@ fn get_next_token_inner(
                 eat_next(stream, pos);
                 return Some((Token::Reserved("--".into()), start_pos));
             }
-            ('-', _) if !state.non_unary => return Some((Token::UnaryMinus, start_pos)),
+            ('-', _) if !state.next_token_cannot_be_unary => {
+                return Some((Token::UnaryMinus, start_pos))
+            }
             ('-', _) => return Some((Token::Minus, start_pos)),
 
             ('*', ')') => {
@@ -1961,7 +1960,7 @@ fn get_identifier(
     start_pos: Position,
     first_char: char,
 ) -> Option<(Token, Position)> {
-    let mut result: smallvec::SmallVec<[char; 8]> = Default::default();
+    let mut result = smallvec::SmallVec::<[char; 8]>::new();
     result.push(first_char);
 
     while let Some(next_char) = stream.peek_next() {
@@ -2025,6 +2024,13 @@ pub fn is_valid_identifier(name: impl Iterator<Item = char>) -> bool {
     first_alphabetic
 }
 
+/// Is a text string a valid scripted function name?
+#[inline(always)]
+#[must_use]
+pub fn is_valid_function_name(name: &str) -> bool {
+    is_valid_identifier(name.chars())
+}
+
 /// Is a character valid to start an identifier?
 #[cfg(feature = "unicode-xid-ident")]
 #[inline(always)]
@@ -2119,6 +2125,10 @@ impl InputStream for MultiInputsStream<'_> {
 
 /// _(internals)_ An iterator on a [`Token`] stream.
 /// Exported under the `internals` feature only.
+///
+/// # Volatile Data Structure
+///
+/// This type is volatile and may change.
 pub struct TokenIterator<'a> {
     /// Reference to the scripting `Engine`.
     pub engine: &'a Engine,
@@ -2126,12 +2136,12 @@ pub struct TokenIterator<'a> {
     pub state: TokenizeState,
     /// Current position.
     pub pos: Position,
-    /// External buffer containing the next character to read, if any.
+    /// Shared object to allow controlling the tokenizer externally.
     pub tokenizer_control: TokenizerControl,
     /// Input character stream.
     pub stream: MultiInputsStream<'a>,
     /// A processor function that maps a token to another.
-    pub map: Option<fn(Token) -> Token>,
+    pub token_mapper: Option<&'a OnParseTokenCallback>,
 }
 
 impl<'a> Iterator for TokenIterator<'a> {
@@ -2225,8 +2235,8 @@ impl<'a> Iterator for TokenIterator<'a> {
         };
 
         // Run the mapper, if any
-        let token = if let Some(map_func) = self.map {
-            map_func(token)
+        let token = if let Some(map_func) = self.token_mapper {
+            map_func(token, pos, &self.state)
         } else {
             token
         };
@@ -2257,9 +2267,9 @@ impl Engine {
     pub fn lex_with_map<'a>(
         &'a self,
         input: impl IntoIterator<Item = &'a &'a str>,
-        map: fn(Token) -> Token,
+        token_mapper: &'a OnParseTokenCallback,
     ) -> (TokenIterator<'a>, TokenizerControl) {
-        self.lex_raw(input, Some(map))
+        self.lex_raw(input, Some(token_mapper))
     }
     /// Tokenize an input text stream with an optional mapping function.
     #[inline]
@@ -2267,7 +2277,7 @@ impl Engine {
     pub(crate) fn lex_raw<'a>(
         &'a self,
         input: impl IntoIterator<Item = &'a &'a str>,
-        map: Option<fn(Token) -> Token>,
+        token_mapper: Option<&'a OnParseTokenCallback>,
     ) -> (TokenIterator<'a>, TokenizerControl) {
         let buffer: TokenizerControl = Default::default();
         let buffer2 = buffer.clone();
@@ -2280,7 +2290,7 @@ impl Engine {
                     max_string_size: self.limits.max_string_size,
                     #[cfg(feature = "unchecked")]
                     max_string_size: None,
-                    non_unary: false,
+                    next_token_cannot_be_unary: false,
                     comment_level: 0,
                     include_comments: false,
                     is_within_text_terminated_by: None,
@@ -2292,7 +2302,7 @@ impl Engine {
                     streams: input.into_iter().map(|s| s.chars().peekable()).collect(),
                     index: 0,
                 },
-                map,
+                token_mapper,
             },
             buffer2,
         )
